@@ -6,7 +6,7 @@ use Lemonldap::NG::Common::Combination::Parser;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_ERROR);
 use Scalar::Util 'weaken';
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 # TODO: See Lib::Wrapper
 extends 'Lemonldap::NG::Portal::Auth::Base';
@@ -94,9 +94,6 @@ sub init {
 # the current user
 sub extractFormInfo {
     my ( $self, $req ) = @_;
-
-    # Get available authentication schemes for this user
-    $self->getStack( $req, 'extractFormInfo' ) or return PE_ERROR;
     return $self->try( 0, 'extractFormInfo', $req );
 }
 
@@ -125,8 +122,10 @@ sub getDisplayType {
     my ( $self, $req ) = @_;
     return $self->conf->{combinationForms}
       if ( $self->conf->{combinationForms} );
-    my ( $nb, $stack ) =
-      ( $req->datas->{combinationTry}, $req->datas->{combinationStack} );
+    my ( $nb, $stack ) = (
+        $req->datas->{dataKeep}->{combinationTry},
+        $req->datas->{combinationStack}
+    );
     my ( $res, $name ) = $stack->[$nb]->[0]->( 'getDisplayType', $req );
     return $res;
 }
@@ -151,15 +150,22 @@ sub getStack {
         $self->logger->error('No authentication scheme for this user');
     }
     @{ $req->datas->{combinationSteps} } = ( @steps, @{ $req->steps } );
-    $req->datas->{combinationTry} = 0;
+    $req->datas->{dataKeep}->{combinationTry} ||= 0;
     return $stack;
 }
 
 # Main running method: launch the next scheme if the current fails
 sub try {
     my ( $self, $type, $subname, $req ) = @_;
-    my ( $nb, $stack ) =
-      ( $req->datas->{combinationTry}, $req->datas->{combinationStack} );
+
+    # Get available authentication schemes for this user if not done
+    unless ( defined $req->datas->{combinationStack} ) {
+        $self->getStack( $req, $subname ) or return PE_ERROR;
+    }
+    my ( $nb, $stack ) = (
+        $req->datas->{dataKeep}->{combinationTry},
+        $req->datas->{combinationStack}
+    );
 
     # If more than 1 scheme is available
     my ( $res, $name );
@@ -172,7 +178,7 @@ sub try {
         if ( $res > PE_OK ) {
             $self->logger->info(
                 qq'Scheme "$name" has return $res, trying next');
-            $req->datas->{combinationTry}++;
+            $req->datas->{dataKeep}->{combinationTry}++;
             $req->steps( [ @{ $req->datas->{combinationSteps} } ] );
             return PE_OK;
         }
@@ -181,7 +187,8 @@ sub try {
         ( $res, $name ) = $stack->[$nb]->[$type]->( $subname, $req );
     }
     $req->sessionInfo->{ [ '_auth', '_userDB' ]->[$type] } = $name;
-    $req->sessionInfo->{_combinationTry} = $req->datas->{combinationTry};
+    $req->sessionInfo->{_combinationTry} =
+      $req->datas->{dataKeep}->{combinationTry};
     if ( $res > 0 ) {
         $self->userLogger->warn( 'All schemes failed'
               . ( $req->user ? ' for user ' . $req->user : '' ) );
