@@ -4,7 +4,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_BADCREDENTIALS);
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 extends 'Lemonldap::NG::Portal::Auth::_WebForm',
   'Lemonldap::NG::Portal::Lib::DBI';
@@ -21,7 +21,13 @@ sub init {
 
 sub authenticate {
     my ( $self, $req ) = @_;
-    return $self->check_password($req) ? PE_OK : PE_BADCREDENTIALS;
+    if ( $self->check_password($req) ) {
+        return PE_OK;
+    }
+    else {
+        $self->setSecurity($req);
+        return PE_BADCREDENTIALS;
+    }
 }
 
 sub authLogout {
