@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_BADCREDENTIALS);
 
 extends qw(Lemonldap::NG::Portal::Auth::_WebForm);
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 # INITIALIZATION
 
@@ -32,8 +32,10 @@ sub init {
 sub authenticate {
     my ( $self, $req ) = @_;
 
-    return PE_BADCREDENTIALS
-      unless ( $req->{user} eq $req->datas->{password} );
+    unless ( $req->{user} eq $req->datas->{password} ) {
+        $self->setSecurity($req);
+        return PE_BADCREDENTIALS;
+    }
 
     PE_OK;
 }
