@@ -9,7 +9,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
 );
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 extends 'Lemonldap::NG::Portal::Auth::Base',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect';
@@ -18,6 +18,7 @@ extends 'Lemonldap::NG::Portal::Auth::Base',
 
 has opList => ( is => 'rw', default => sub { [] } );
 has opNumber => ( is => 'rw', default => 0 );
+has path     => ( is => 'rw', default => 'oauth2' );
 
 # INITIALIZATION
 
@@ -57,6 +58,16 @@ sub init {
             class => "openidconnect",
           };
     }
+    $self->addRouteFromConf(
+        'Unauth',
+        oidcServiceMetaDataFrontChannelURI => 'alreadyLoggedOut',
+        oidcServiceMetaDataBackChannelURI  => 'backLogout',
+    );
+    $self->addRouteFromConf(
+        'Auth',
+        oidcServiceMetaDataFrontChannelURI => 'frontLogout',
+        oidcServiceMetaDataBackChannelURI  => 'backLogout',
+    );
     $self->opList( [@list] );
     return 1;
 }
@@ -309,4 +320,28 @@ sub getDisplayType {
     return "logo";
 }
 
+sub alreadyLoggedOut {
+    my ( $self, $req ) = @_;
+    $self->userLogger->info(
+        'Front-channel logout request for an already logged out user');
+    my $img = $self->conf->{staticPrefix} . '/common/icons/ok.png';
+
+    # No need to protect this frame
+    $req->frame(1);
+    my $frame = qq'<html><body><img src="$img"></body></html>';
+    return [
+        200,
+        [ 'Content-Type' => 'text/html', 'Content-Length' => length($frame) ],
+        [$frame]
+    ];
+}
+
+sub frontLogout {
+    my ( $self, $req ) = @_;
+}
+
+sub backLogout {
+    my ( $self, $req ) = @_;
+}
+
 1;
