@@ -11,7 +11,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 
 extends qw(Lemonldap::NG::Portal::Auth::_WebForm);
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 # PROPERTIES
 
@@ -48,11 +48,15 @@ sub init {
 sub authenticate {
     my ( $self, $req ) = @_;
     $self->initRadius unless ( $self->radius );
-    return PE_RADIUSCONNECTFAILED unless ( $self->radius );
+    unless ( $self->radius ) {
+        $self->setSecurity($req);
+        return PE_RADIUSCONNECTFAILED;
+    }
 
     my $res = $self->radius->check_pwd( $req->user, $req->datas->{password} );
     unless ( $res == 1 ) {
         $self->userLogger->warn("Unable to authenticate $req->{user} !");
+        $self->setSecurity($req);
         return PE_BADCREDENTIALS;
     }
     return PE_OK;
