@@ -18,7 +18,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_TOKENEXPIRED
 );
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 extends 'Lemonldap::NG::Portal::Auth::Base';
 
@@ -82,16 +82,7 @@ sub extractFormInfo {
 
     # If form seems empty
     if ( $res != PE_OK ) {
-
-        # If captcha is enable, prepare it
-        if ( $self->captcha ) {
-            $self->captcha->setCaptcha($req);
-        }
-
-        # Else get token
-        elsif ( $self->ott ) {
-            $self->ott->setToken($req);
-        }
+        $self->setSecurity($req);
         return $res;
     }
 
@@ -100,6 +91,7 @@ sub extractFormInfo {
         my $token;
         unless ( $token = $req->param('token') ) {
             $self->userLogger->error('Authentication tried without token');
+            $self->ott->setToken($req);
             return PE_NOTOKEN;
         }
         if ( $self->captcha ) {
@@ -156,4 +148,17 @@ sub getDisplayType {
     return "standardform";
 }
 
+sub setSecurity {
+    my($self,$req) = @_;
+    # If captcha is enable, prepare it
+    if ( $self->captcha ) {
+        $self->captcha->setCaptcha($req);
+    }
+
+    # Else get token
+    elsif ( $self->ott ) {
+        $self->ott->setToken($req);
+    }
+}
+
 1;
