@@ -3,6 +3,7 @@ package Lemonldap::NG::Portal::Issuer::OpenIDConnect;
 use strict;
 use JSON;
 use Mouse;
+use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADURL
   PE_CONFIRM
@@ -12,7 +13,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
 );
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::OpenIDConnect',
@@ -164,7 +165,7 @@ sub run {
                 $self->returnRedirectError(
                     $req,
                     $oidc_request->{'redirect_uri'},
-                    "invalid_request",
+                    'invalid_request',
                     "scope required",
                     undef,
                     $oidc_request->{'state'},
@@ -176,7 +177,7 @@ sub run {
                 return $self->returnRedirectError(
                     $req,
                     $oidc_request->{'redirect_uri'},
-                    "invalid_request",
+                    'invalid_request',
                     "client_id required",
                     undef,
                     $oidc_request->{'state'},
@@ -188,7 +189,7 @@ sub run {
                 $self->logger->error("Nonce is required for implicit flow");
                 return $self->returnRedirectError(
                     $req, $oidc_request->{'redirect_uri'},
-                    "invalid_request", "nonce required",
+                    'invalid_request', "nonce required",
                     undef, $oidc_request->{'state'}, 1
                 );
             }
@@ -290,7 +291,7 @@ sub run {
                 return $self->returnRedirectError(
                     $req,
                     $oidc_request->{'redirect_uri'},
-                    "invalid_request",
+                    'invalid_request',
                     "client_id $client_id unknown",
                     undef,
                     $oidc_request->{'state'},
@@ -355,7 +356,7 @@ sub run {
                     return $self->returnRedirectError(
                         $req,
                         $oidc_request->{'redirect_uri'},
-                        "invalid_request",
+                        'invalid_request',
                         "current user do not match id_token_hint sub",
                         undef,
                         $oidc_request->{'state'},
@@ -800,8 +801,18 @@ qq'<h3 trspan="oidcConsent,$display_name">The application $display_name would li
             # Ask consent for logout
             if ( $req->param('confirm') ) {
                 if ( $req->param('confirm') == 1 ) {
-                    my $apacheSession = $self->p->getApacheSession( $req->id );
-                    $self->p->_deleteSession( $req, $apacheSession );
+                    $req->steps(
+                        [
+                            @{ $self->p->beforeLogout }, 'authLogout',
+                            'deleteSession'
+                        ]
+                    );
+                    my $err = $req->error( $self->p->process($req) );
+                    if ( $err > 0 ) {
+                        $self->logger->error(
+                            "Logout process returns error code $err");
+                        return PE_ERROR;
+                    }
                 }
 
                 if ($post_logout_redirect_uri) {
@@ -842,7 +853,7 @@ sub token {
         $self->logger->error(
 "No authentication provided to get token, or authentication type not supported"
         );
-        return $self->p->sendError( $req, 'unauthorized_client', 401 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     # Verify that client_id is registered in configuration
@@ -851,7 +862,7 @@ sub token {
     unless ($rp) {
         $self->userLogger->error(
             "No registered Relying Party found with client_id $client_id");
-        return $self->p->sendError( $req, "unauthorized_client", 403 );
+        return $self->p->sendError( $req, 'invalid_request', 400 );
     }
     else {
         $self->logger->debug("Client id $client_id match RP $rp");
@@ -862,7 +873,7 @@ sub token {
         ->{oidcRPMetaDataOptionsClientSecret} )
     {
         $self->logger->error("Wrong credentials for $rp");
-        return $self->p->sendError( "access_denied", 403 );
+        return $self->p->sendError( 'invalid_request', 400 );
     }
 
     # Get code session
@@ -874,7 +885,7 @@ sub token {
 
     unless ($codeSession) {
         $self->logger->error("Unable to find OIDC session $code");
-        $self->p->sendError( $req, "invalid_grant", 400 );
+        $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     # Check we have the same redirect_uri value
@@ -882,7 +893,7 @@ sub token {
     {
         $self->userLogger->error( "Provided redirect_uri is different from "
               . $codeSession->{redirect_uri} );
-        $self->p->sendError( $req, "invalid_request", 400 );
+        $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     # Get user identifier
@@ -894,7 +905,7 @@ sub token {
         $self->userLogger->error(
             "Unable to find user session linked to OIDC session $code");
         $codeSession->remove();
-        $self->p->sendError( $req, "invalid_request", 400 );
+        $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     my $user_id_attribute =
@@ -920,7 +931,7 @@ sub token {
         $self->userLogger->error(
             "Unable to create OIDC session for access_token");
         $codeSession->remove();
-        $self->p->sendError( $req, "invalid_request", 400 );
+        $self->p->sendError( $req, 'invalid_request', 400 );
     }
 
     my $access_token = $accessTokenSession->id;
@@ -973,6 +984,10 @@ sub token {
         id_token     => $id_token,
     };
 
+    my $cRP = join ',', $rp, $apacheSession->data->{_oidcConnectedRP} || '';
+    $self->p->updateSession( $req, { _oidcConnectedRP => $cRP },
+        $apacheSession->id );
+
     $self->logger->debug("Send token response");
 
     $codeSession->remove();
@@ -988,8 +1003,8 @@ sub userInfo {
 
     unless ($access_token) {
         $self->logger->error("Unable to get access_token");
-        return $self->returnBearerError( "invalid_request",
-            "Access token not found in request" );
+        return $self->returnBearerError( 'invalid_request',
+            "Access token not found in request", 401 );
     }
 
     $self->logger->debug("Received Access Token $access_token");
@@ -999,8 +1014,8 @@ sub userInfo {
     unless ($accessTokenSession) {
         $self->userLogger->error(
             "Unable to get access token session for id $access_token");
-        return $self->returnBearerError( "invalid_token",
-            "Access Token not found or expired" );
+        return $self->returnBearerError( 'invalid_request',
+            'Invalid request', 401 );
     }
 
     # Get access token session data
@@ -1011,7 +1026,8 @@ sub userInfo {
     my $userinfo_response =
       $self->buildUserInfoResponse( $scope, $rp, $user_session_id );
     unless ($userinfo_response) {
-        return $self->p->sendError( $req, "invalid_request", 400 );
+        return $self->returnBearerError( 'invalid_request', 'Invalid request',
+            401 );
     }
 
     my $userinfo_sign_alg = $self->conf->{oidcRPMetaDataOptions}->{$rp}
@@ -1113,6 +1129,7 @@ sub registration {
 
     $conf->{cfgAuthor}   = "OpenID Connect Registration ($client_name)";
     $conf->{cfgAuthorIP} = $source_ip;
+    $conf->{cfgVersion}  = $VERSION;
 
     $conf->{oidcRPMetaDataExportedVars}->{$rp} = {};
     $conf->{oidcRPMetaDataOptions}->{$rp}->{oidcRPMetaDataOptionsClientID} =
@@ -1210,25 +1227,43 @@ sub badAuthRequest {
 # Nothing to do here
 sub logout {
     my ( $self, $req ) = @_;
+    if ( my $s = $req->userData->{_oidcConnectedRP} ) {
+        my @rps = grep /\w/, split( ',', $s );
+        foreach my $rp (@rps) {
+            my $rpConf = $self->conf->{oidcRPMetaDataOptions}->{$rp};
+            unless ($rpConf) {
+                $self->logger->error("Unknown RP $rp");
+                return PE_ERROR;
+            }
+            if ( my $url = $rpConf->{oidcRPMetaDataOptionsLogoutUrl} ) {
+                if ( $rpConf->{oidcRPMetaDataOptionsLogoutType} eq 'front' ) {
+                    if ( $rpConf->{oidcRPMetaDataOptionsLogoutSessionRequired} )
+                    {
+                        my $user_id_attribute =
+                          $self->conf->{oidcRPMetaDataOptions}->{$rp}
+                          ->{oidcRPMetaDataOptionsUserIDAttr}
+                          || $self->conf->{whatToTrace};
+                        my $user_id = $req->{sessionInfo}->{$user_id_attribute};
+                        $url .= ( $url =~ /\?/ ? '&' : '?' )
+                          . build_urlencoded(
+                            iss => $self->conf->{oidcServiceMetaDataIssuer},
+                            sid => $user_id
+                          );
+                    }
+                    $req->info( qq'<iframe src="$url" class="noborder">'
+                          . '</iframe>' );
+                }
+                else {
+                    # TODO
+                }
+            }
+        }
+    }
     return PE_OK;
 }
 
 # Internal methods
 
-sub addRouteFromConf {
-    my ( $self, $type, %subs ) = @_;
-    my $adder = "add${type}Route";
-    foreach ( keys %subs ) {
-        my $sub  = $subs{$_};
-        my $path = $self->conf->{$_};
-        unless ($path) {
-            $self->logger->error("$_ parameter not defined");
-            next;
-        }
-        $self->$adder( $self->path => { $path => $sub }, [ 'GET', 'POST' ] );
-    }
-}
-
 sub metadata {
     my ( $self, $req ) = @_;
     my $issuerDBOpenIDConnectPath = $self->conf->{issuerDBOpenIDConnectPath};
@@ -1262,6 +1297,12 @@ sub metadata {
             authorization_endpoint => $baseUrl . $authorize_uri,
             end_session_endpoint   => $baseUrl . $endsession_uri,
             check_session_iframe   => $baseUrl . $checksession_uri,
+
+            # Logout capabilities
+            backchannel_logout_supported          => JSON::true,
+            backchannel_logout_session_supported  => JSON::true,
+            frontchannel_logout_supported         => JSON::true,
+            frontchannel_logout_session_supported => JSON::true,
             (
                 $self->conf->{oidcServiceAllowDynamicRegistration}
                 ? ( registration_endpoint => $baseUrl . $registration_uri )
