@@ -3,7 +3,6 @@ package Lemonldap::NG::Portal::Issuer::SAML;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_INFO
   PE_OK
   PE_SAML_ART_ERROR
   PE_SAML_DESTINATION_ERROR
@@ -14,7 +13,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SAML_UNKNOWN_ENTITY
 );
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
@@ -774,6 +773,7 @@ sub run {
 
             # Artifact
             # Choose method
+            no strict 'subs';
             if (   $artifact
                 or $protocolProfile ==
                 Lasso::Constants::LOGIN_PROTOCOL_PROFILE_BRWS_ART )
@@ -1320,6 +1320,7 @@ sub sloRelaySoap {
     }
 
     # Send the logout request
+    no strict 'subs';
     my ( $rstatus, $rmethod, $rinfo ) =
       $self->sendLogoutRequestToProvider( $req, $logout, $providerID,
         Lasso::Constants::HTTP_METHOD_SOAP,
@@ -1874,12 +1875,12 @@ sub attributeServer {
 
 sub imgok {
     my ( $self, $req, ) = @_;
-    return $self->sendImage( $req, 'ok.png' );
+    return $self->sendImage( $req, 'icons/ok.png' );
 }
 
 sub imgnok {
     my ( $self, $req, ) = @_;
-    return $self->sendImage( $req, 'warning.png' );
+    return $self->sendImage( $req, 'icons/warning.png' );
 }
 
 sub sendImage {
