@@ -5,7 +5,7 @@ use Mouse;
 
 extends 'Lemonldap::NG::Portal::Lib::Wrapper';
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 has modules => ( is => 'rw', default => sub { {} } );
 
@@ -76,7 +76,7 @@ sub name {
     unless ($req) {
         return 'Choice';
     }
-    my $n = ref( $req->datas->{enableMods}->[0] );
+    my $n = ref( $req->datas->{enabledMods}->[0] );
     $n =~ s/^Lemonldap::NG::Portal::(?:(?:UserDB|Auth)::)?//;
     return $n;
 }
