@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -1372,6 +1372,20 @@ sub decode_base64url {
     return decode_base64($s);
 }
 
+sub addRouteFromConf {
+    my ( $self, $type, %subs ) = @_;
+    my $adder = "add${type}Route";
+    foreach ( keys %subs ) {
+        my $sub  = $subs{$_};
+        my $path = $self->conf->{$_};
+        unless ($path) {
+            $self->logger->error("$_ parameter not defined");
+            next;
+        }
+        $self->$adder( $self->path => { $path => $sub }, [ 'GET', 'POST' ] );
+    }
+}
+
 1;
 
 __END__
@@ -1525,6 +1539,11 @@ Build Logout Request URI
 
 Build Logout Response URI 
 
+=head2 addRouteFromConf
+
+Build a Lemonldap::NG::Common::PSGI::Router route from OIDC configuration
+attribute
+
 =head1 SEE ALSO
 
 L<Lemonldap::NG::Portal::AuthOpenIDConnect>, L<Lemonldap::NG::Portal::UserDBOpenIDConnect>
