@@ -19,7 +19,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SAML_SLO_ERROR
 );
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 # PROPERTIES
 
@@ -237,7 +237,8 @@ sub loadIDPs {
         }
 
         # Store IDP entityID and Organization Name
-        my ($entityID) = ( $idp_metadata =~ /entityID="(.+?)"/si );
+        my ( $tmp, $entityID ) =
+          ( $idp_metadata =~ /entityID=(['"])(.+?)\1/si );
         my $name = $self->getOrganizationName( $self->lassoServer, $entityID )
           || ucfirst($_);
         $self->idpList->{$entityID}->{confKey} = $_;
@@ -310,7 +311,8 @@ sub loadSPs {
         }
 
         # Store SP entityID and Organization Name
-        my ($entityID) = ( $sp_metadata =~ /entityID="(.+?)"/i );
+        my ( $tmp, $entityID ) =
+          ( $sp_metadata =~ /entityID=(['"])(.+?)\1/si );
         my $name = $self->getOrganizationName( $self->lassoServer, $entityID )
           || ucfirst($_);
         $self->spList->{$entityID}->{confKey} = $_;
