@@ -2,7 +2,7 @@
 # into "plugins" list in lemonldap-ng.ini, section "portal"
 package Lemonldap::NG::Portal::Main::Plugins;
 
-our $VERSION = '1.9.99_03';
+our $VERSION = '1.9.991_01';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -22,7 +22,7 @@ our @pList = (
     u2fSelfRegistration        => '::Register::U2F',
     notification               => '::Plugins::Notifications',
     portalCheckLogins          => '::Plugins::History',
-    sTokenService              => '::Plugins::ServiceTokenServer',
+    stayConnected              => '::Plugins::StayConnected',
 );
 
 ##@method list enabledPlugins
